;
;BeOS 4.5.2 NT bootstrap routine
;

   org 0                     ;use 'bin' format
   xor ax,ax                 ;clear ax
   mov ss,ax                 ;set ss = ds
   mov sp,0x7c00             ;point at address expected by boot loader  
   cld                       ;set up string operations
   push ax                   ;save origin on stack
   pop ds                    ;load [sp] into ds and incr stack ptr
   push ds                   ;restore sp
   pop es                    ;load [sp] into es
   mov si,sp                 ;src now points to org + 2
   mov di,0x7e00             ;set dest to 0x7c00 + 512 bytes
   mov cx,0x100              ;set the word counter to 256
   cld                       ;
   rep movsw                 ;mov word [es:di]->[ds:si] cx times
   jmp 0x21a                 ;continue next line in new mem block

                             ;now we're bootstrapped to base 0x7e00

   mov dl,[0x7ff9]           ;move [0x7e00+0x1f9] (0x1) into dl
   mov eax,[0x7ffa]          ;move [0x7e99+0x1fa] (0x003F) into eax
   mov bx,0x7c00             ;load base with boot loader address
   push es                   ;
   push bx                   ;
   mov [0xa1],eax            ;save 0x003F into scratch memory at [0xa1]
   push es                   ;save registers
   push bx                   ;
   push dx                   ;
   push eax                  ;
   mov ah,0x8                ;int 13 func 8
   int 0x13                  ;disk - read drive parameters
                             ;dh = max head #, dl = # drives
                             ;ch = low 8 bits of max cylinder #
                             ;cl = max sector # (bits 5-0) & high bits of max cyl # (7-6)

   mov al,cl                 ;al = max sector # & high bits of max cylinder #
   and ax,0x3f               ;mask first 5 bits
   mov bx,ax                 ;save max sector # in bx
   mov dl,0x0                ;set # drives to 0
   xchg dl,dh                ;dh = 0, dl = max head #
   inc dx                    ;make max heads 1 based
   mul dx                    ;ax * dx->[dx:ax] ([dx:ax] = sect/track * heads = sect/cyl)
   xchg ax,cx                ;move sectors/cylinder count to cx
   pop ax                    ;ax = 0x3F (max permissible sectors/cyl = 64)
   pop dx                    ;dx = 0x1
   jcxz message              ;iff cx = 0, jump to message (we give up, system says no disk)
   cmp dx,cx                 ;else if dx = cx (is this drive 1 ?)
   jnc test_sig              ;jump to test_sig
   div cx                    ;ax = dx:ax / cx (div by 64)
   xchg ax,cx                ;move new sectors/cylinder count into cx
   xchg ax,dx                ;get [cx] into dx
   div bl                    ;bl = ax / bl
   xchg cl,ch                ;reverse order
   ror cl,0x2                ;cl = cl / 4
   or cl,ah                  ;or - leaves result in cl
   inc cx                    ;
   pop dx                    ;
   mov dh,al                 ;
   pop bx                    ;
   pop es                    ;
   mov di,0x5                ;set number of retries

retry

   mov ax,0x201              ;int 13 func 2 (al = 1 sector)
   int 0x13                  ;disk - read sector(s) into memory
   jnc this_one              ;if successful, jump to this_one
   xor ax,ax                 ;int 13 func 0
   int 0x13                  ;reset disk system, seek track 0
   dec di                    ;decrement retries
   jnz retry                 ;if di > 0 jump to retry

message

   mov si,0x7ea9             ;point at disk_msg
   xor bx,bx                 ;clear base
   mov ah,0xe                ;0xe - shift out

putch

   lodsb                     ;load character at si into al
   int 0x10                  ;write to screen
   cmp al,0x2e               ;look for '.' in string
   jnz putch                 ;if no match, get next character
   xchg ax,bx                ;int 16 func 0
   int 0x16                  ;get keystroke
   int 0x19                  ;reboot

test_sig

   mov ah,0x42               ;int 13 func 42
   mov si,0x7e99             ;point at disk address packet, offset 0x99 (0x7e99)
   int 0x13                  ;disk - extended read
   jc message                ;if carry flag, jump message

this_one

   mov bx,[0x7dfe]           ;read original 0xaa55 signature from [0x7c00+0x1fe]
   mov si,0x7ecd             ;point at load_msg
   cmp bx,0xaa55             ;valid signature ?
   jnz message               ;
   retf                      ;pop ip/eip, pop cs and return to caller


;disk address packet, offset 0x99 (0x7e99)
   db 0x10                   ;size of packet
   db 0x00                   ;reserved
   dw 0x0001                 ;# blocks to transfer
   dw 0x7C00                 ;dw -> transfer buffer

;scratch memory, offset 0xa0 (0x7ea0)
   times 5 dw 0x0000         ;qword starting absolute block #


;offset 0xA9 (0x7ea9)

disk_msg   db 'Disk error reading BeOS boot sector.'


;offset 0xCD (0x7ecd)

load_msg   db 'Boot loader not found on loaded partition.'


   times 505-($-$$) db 0      ;pad file with zeros
   dw 0x3F81                  ;HDD 1, address 3F
   times 3 db 0               ;continuation of boot address
   dw 0xAA55                  ;the boot signature (0x7ffe)
